@@ -1,5 +1,4 @@
 import sys
-from collections import defaultdict
 from copy import deepcopy
 from dataclasses import dataclass, asdict
 from logging import getLogger, WARNING
@@ -7,6 +6,7 @@ from logging import getLogger, WARNING
 import anyconfig
 import click
 
+from .command import CwsMultiCommands
 from .error import CwsClientError
 from ..config import DEFAULT_PROJECT_DIR, DEFAULT_WORKSPACE
 from ..utils import import_attr, get_system_info
@@ -16,10 +16,12 @@ from ..version import __version__
 @click.group()
 @click.version_option(version=__version__, message=f'%(prog)s %(version)s, {get_system_info()}')
 @click.option('-p', '--project-dir', default=DEFAULT_PROJECT_DIR,
-              help='The project directory path (absolute or relative). Defaults to CWD')
+              help=f"The project directory path (absolute or relative) [default to '{DEFAULT_PROJECT_DIR}'].")
+@click.option('-c', '--config_file', help="Configuration file path [path from project dir].")
 @click.option('-m', '--module', help="Filename of your microservice python source file.")
 @click.option('-s', '--service', help="Coworks application in the source file.")
-@click.option('-w', '--workspace', default=DEFAULT_WORKSPACE, help="Application stage.")
+@click.option('-w', '--workspace', default=DEFAULT_WORKSPACE,
+              help=f"Application stage [default to '{DEFAULT_WORKSPACE}'].")
 @click.pass_context
 def client(*args, **kwargs):
     ...
@@ -30,33 +32,23 @@ def invoke(ctx):
     try:
         args = ctx.args
         protected_args = ctx.protected_args
-
         command_name = protected_args[0] if protected_args else None
 
-        project_dir = ctx.params.get('project_dir')
-        workspace = ctx.params.get('workspace')
-        module = ctx.params.get('module')
-        service = ctx.params.get('service')
-
-        project_config = ProjectConfig(project_dir)
-        if service:
-            services = [(module, service)]
-        else:
-            services = project_config.all_services(module)
-
-        if not services:
+        # get project options
+        cws_options = CwsClientOptions(ctx.params)
+        if not cws_options.services:
             sys.stderr.write(str("Nothing to execute as no service defined."))
             sys.exit(1)
 
         # Iterates over the declared services in project configuration file
-        commands_to_be_executed = defaultdict(list)
-        for module, service in services:
+        commands_to_be_executed = CwsMultiCommands()
+        for module, service in cws_options.services:
             ctx.args = list(args)
             ctx.protected_args = protected_args
-            service_config = project_config.get_service_config(module, service, workspace)
 
-            # Get command from the microservice
-            handler = get_handler(project_dir, module, service)
+            # Get command from the microservice description
+            handler = cws_options.get_handler(module, service)
+            service_config = cws_options.get_service_config(module, service)
             command = service_config.get_command(command_name, handler)
             if not command:
                 raise CwsClientError(f"Undefined command {command_name}.\n")
@@ -68,26 +60,73 @@ def invoke(ctx):
                 cmd_opt = next(x for x in cmd_opts if x.name == opt_key)
                 client_options[opt_key] = cmd_opt.type(opt_value)
 
-            execution_params = {**command_options, **client_options}
-            command.make_context(command.name, execution_params)
-            commands_to_be_executed[type(command)].append((command, execution_params))
-        for command_class, execution_params in commands_to_be_executed.items():
-            command_class.multi_execute(project_dir, workspace, execution_params)
+            # Adds command and global options
+            options = {**command_options, **client_options}
+            command.make_context(command.name, options)
+            commands_to_be_executed.append(client_options, command, options)
+
+        # Executes all commands
+        project_dir = cws_options.project_dir
+        workspace = cws_options.workspace
+        client_options = commands_to_be_executed.client_options
+        for command_class, execution_context in commands_to_be_executed.items():
+            command_class.multi_execute(project_dir, workspace, client_options, execution_context)
     except CwsClientError as client_err:
-        sys.stderr.write(client_err.msg)
+        sys.stderr.write(f"Error in command: {client_err.msg}")
         sys.exit(1)
     except Exception as e:
-        sys.stderr.write(str(e))
+        sys.stderr.write(f"Error in command: {str(e)}")
         sys.exit(1)
 
 
 client.invoke = invoke
 
 
+@dataclass
+class CwsClientOptions:
+    """Client options defined from click command."""
+    project_dir: str
+    workspace: str
+    module: str
+    service: str
+    config_file: str
+
+    def __init__(self, params):
+        self.project_dir = params.get('project_dir')
+        self.workspace = params.get('workspace')
+        self.module = params.get('module')
+        self.service = params.get('service')
+        self.config_file = params.get('config_file') or "project"
+        self.project_config = ProjectConfig(self.project_dir, self.config_file)
+
+    @property
+    def services(self):
+        """Returns the list of services defined from the client optons"""
+        if self.service:
+            return [(self.module, self.service)]
+        return self.project_config.all_services(self.module)
+
+    def get_handler(self, module, service):
+        # Load microservice handler
+        try:
+            return import_attr(module, service, cwd=self.project_dir)
+        except AttributeError as e:
+            raise CwsClientError(f"Module '{module}' has no microservice {service} : {str(e)}\n")
+        except ModuleNotFoundError as e:
+            raise CwsClientError(f"The module '{module}' is not defined in {self.project_dir} : {str(e)}\n")
+        except Exception as e:
+            raise CwsClientError(f"Error {e} when loading module '{module}'\n")
+
+    def get_service_config(self, module, service, workspace=None):
+        """Returns the microserrvice's configuration."""
+        workspace = workspace or self.workspace
+        return ServiceConfig(self.project_config, module, service, workspace)
+
+
 class ProjectConfig:
     """Class for the project configuration file."""
 
-    def __init__(self, project_dir, file_name="cws.project", file_suffix=".yml"):
+    def __init__(self, project_dir, file_name, file_suffix=".cws.yml"):
         from pathlib import Path
         self.project_dir = project_dir
         self.params = {}
@@ -211,17 +250,5 @@ def main():
     return client(obj={})
 
 
-def get_handler(project_dir, module, service):
-    # Load microservice handler
-    try:
-        return import_attr(module, service, cwd=project_dir)
-    except AttributeError as e:
-        raise CwsClientError(f"Module '{module}' has no microservice {service} : {str(e)}\n")
-    except ModuleNotFoundError as e:
-        raise CwsClientError(f"The module '{module}' is not defined in {project_dir} : {str(e)}\n")
-    except Exception as e:
-        raise CwsClientError(f"Error {e} when loading module '{module}'\n")
-
-
 if __name__ == "__main__":
     main()
