@@ -1,6 +1,4 @@
-class CwsError(Exception):
-    def __init__(self, message):
-        self.msg = message
+from ..error import CwsError
 
 
 class CwsClientError(CwsError):
