@@ -2,22 +2,23 @@ import cgi
 import inspect
 import io
 import json
-import os
 import traceback
 import urllib
 from functools import update_wrapper, partial
 
-import boto3
 from aws_xray_sdk.core import xray_recorder
 from botocore.exceptions import BotoCoreError
 from chalice import AuthResponse, BadRequestError, Response
 from requests_toolbelt.multipart import MultipartDecoder
 
+from coworks import aws
 from coworks.error import CwsError
 from .utils import HTTP_METHODS, class_auth_methods, class_http_methods, trim_underscores, make_absolute
 
 
 class EntryPoint:
+    """An entry point is an API entry defined on a component, with a specific authorization function and
+    is response function."""
 
     def __init__(self, component, auth, fun):
         self.component = component
@@ -65,10 +66,10 @@ class CoworksMixin:
         # To register a function, use the :meth:`handle_exception` decorator.
         self.handle_exception_funcs = []
 
-        self.aws_s3_sfn_data_session = AwsS3Session(env_var_access_key="AWS_RUN_ACCESS_KEY_ID",
+        self.aws_s3_sfn_data_session = aws.AwsS3Session(env_var_access_key="AWS_RUN_ACCESS_KEY_ID",
                                                     env_var_secret_key="AWS_RUN_SECRET_KEY",
                                                     env_var_region="AWS_RUN_REGION")
-        self.aws_s3_form_data_session = AwsS3Session(env_var_access_key="AWS_FORM_DATA_ACCESS_KEY_ID",
+        self.aws_s3_form_data_session = aws.AwsS3Session(env_var_access_key="AWS_FORM_DATA_ACCESS_KEY_ID",
                                                      env_var_secret_key="AWS_FORM_DATA_SECRET_KEY",
                                                      env_var_region="AWS_FORM_DATA_REGION")
 
@@ -118,13 +119,18 @@ class CoworksMixin:
         self.handle_exception_funcs.append(f)
         return f
 
-    def _init_routes(self, app, *, url_prefix='', authorizer=None, hide_routes=False):
-        # External authorizer has priority (forced)
-        if authorizer is None:
+    def _init_routes(self, app, *, url_prefix='', authorizer=None, hide_routes=None):
+        """ Creates all routes for a microservice.
+        :param authorizer is the default global authorization function.
+        :param hide_routes list of routes to be hidden.
+        """
+
+        # Global authorization function may be redefined
+        if authorizer is not None:
+            auth = authorizer
+        else:
             auth_fun = app.config.auth if app.config.auth else class_auth_methods(self)
             auth = self._create_auth_proxy(auth_fun) if auth_fun else None
-        else:
-            auth = authorizer
 
         # Adds entrypoints
         methods = class_http_methods(self)
@@ -138,6 +144,7 @@ class CoworksMixin:
                 name = trim_underscores(name)  # to allow several functions with same route but different args
                 name = name.replace('_', '/')
                 route = f"{url_prefix}/{name}" if url_prefix else f"{name}"
+            entry_path = route
 
             # Get parameters
             args = inspect.getfullargspec(func).args[1:]
@@ -146,18 +153,20 @@ class CoworksMixin:
             if defaults:
                 len_defaults = len(defaults)
                 for index, arg in enumerate(args[:-len_defaults]):
+                    entry_path = entry_path + f"/{{{arg}}}" if entry_path else f"{{{arg}}}"
                     route = route + f"/{{_{index}}}" if route else f"{{_{index}}}"
                 kwarg_keys = args[-len_defaults:]
             else:
                 for index, arg in enumerate(args):
+                    entry_path = entry_path + f"/{{{arg}}}" if entry_path else f"{{{arg}}}"
                     route = route + f"/{{_{index}}}" if route else f"{{_{index}}}"
                 kwarg_keys = {}
 
             proxy = self._create_rest_proxy(func, kwarg_keys, args, varkw)
 
-            # complete all entries
+            # Creates the entry (TODO: autorization may be redefined with decorator)
             route = make_absolute(route)
-            app.entries[route][method.upper()] = EntryPoint(self, auth, func)
+            app.entries[entry_path][method.upper()] = EntryPoint(self, auth, func)
             if not hide_routes and not getattr(func, '__cws_hidden', False):
                 app.route(f"{route}", methods=[method.upper()], authorizer=auth, cors=app.config.cors,
                           content_types=list(app.config.content_type))(proxy)
@@ -400,67 +409,6 @@ def _convert_response(resp):
     return resp
 
 
-class Boto3Mixin:
-
-    def __init__(self, service='s3', profile_name=None, env_var_access_key='aws_access_key_id',
-                 env_var_secret_key='aws_secret_access_key', env_var_region='aws_region',
-                 **kwargs):
-        super().__init__(**kwargs)
-        self.__session__ = self.__client__ = None
-        self.__service = service
-        self.__env_var_access_key = env_var_access_key
-        self.__env_var_secret_key = env_var_secret_key
-        self.__env_var_region = env_var_region
-
-        self.__profile_name = profile_name
-
-    @property
-    def aws_access_key(self):
-        value = os.getenv(self.__env_var_access_key)
-        if not value:
-            raise EnvironmentError(f"{self.__env_var_access_key} not defined in environment")
-        return value
-
-    @property
-    def aws_secret_access_key(self):
-        value = os.getenv(self.__env_var_secret_key)
-        if not value:
-            raise EnvironmentError(f"{self.__env_var_secret_key} not defined in environment")
-        return value
-
-    @property
-    def region_name(self):
-        value = os.getenv(self.__env_var_region)
-        if not value:
-            raise EnvironmentError(f"{self.__env_var_region} not defined in environment")
-        return value
-
-    @property
-    def client(self):
-        if self.__client__ is None:
-            self.__client__ = self.__session.client(self.__service)
-        return self.__client__
-
-    @property
-    def __session(self):
-        if self.__session__ is None:
-            region_name = self.region_name
-            if self.__profile_name is not None:
-                try:
-                    self.__session__ = boto3.Session(profile_name=self.__profile_name, region_name=region_name)
-                except Exception:
-                    raise CwsError(f"Cannot create session for profile {self.__profile_name} and region {region_name}")
-            else:
-                access_key = self.aws_access_key
-                secret_key = self.aws_secret_access_key
-                try:
-                    self.__session__ = boto3.Session(access_key, secret_key, region_name=region_name)
-                except Exception:
-                    raise CwsError(
-                        f"Cannot create session for key {access_key}, secret {secret_key} and region {region_name}")
-        return self.__session__
-
-
 class FileParam:
 
     def __init__(self, file, mime_type):
@@ -471,15 +419,3 @@ class FileParam:
         if self.mime_type:
             return f'FileParam({self.file.name}, {self.mime_type})'
         return f'FileParam({self.file.name})'
-
-
-class AwsS3Session(Boto3Mixin):
-
-    def __init__(self, **kwargs):
-        super().__init__(service='s3', **kwargs)
-
-
-class AwsSFNSession(Boto3Mixin):
-
-    def __init__(self, **kwargs):
-        super().__init__(service='stepfunctions', **kwargs)
